@@ -32,13 +32,11 @@ appropriate to describe the dataset, or when features are correlated.
 We will use data from the `"Current Population Survey"
 <https://www.openml.org/d/534>`_ from 1985 to predict wage as a function of
 various features such as experience, age, or education.
-
-.. contents::
-   :local:
-   :depth: 1
-
 """
 
+# Authors: The scikit-learn developers
+# SPDX-License-Identifier: BSD-3-Clause
+
 # %%
 import matplotlib.pyplot as plt
 import numpy as np
@@ -58,8 +56,8 @@ from sklearn.datasets import fetch_openml
 survey = fetch_openml(data_id=534, as_frame=True)
 
 # %%
-# Then, we identify features `X` and targets `y`: the column WAGE is our
-# target variable (i.e., the variable which we want to predict).
+# Then, we identify features `X` and target `y`: the column WAGE is our
+# target variable (i.e. the variable which we want to predict).
 
 X = survey.data[survey.feature_names]
 X.describe(include="all")
@@ -91,11 +89,11 @@ from sklearn.model_selection import train_test_split
 X_train, X_test, y_train, y_test = train_test_split(X, y, random_state=42)
 
 # %%
-# First, let's get some insights by looking at the variable distributions and
+# First, let's get some insights by looking at the variables' distributions and
 # at the pairwise relationships between them. Only numerical
 # variables will be used. In the following plot, each dot represents a sample.
 #
-#   .. _marginal_dependencies:
+# .. _marginal_dependencies:
 
 train_dataset = X_train.copy()
 train_dataset.insert(0, "WAGE", y_train)
@@ -109,7 +107,7 @@ _ = sns.pairplot(train_dataset, kind="reg", diag_kind="kde")
 #
 # The WAGE is increasing when EDUCATION is increasing.
 # Note that the dependence between WAGE and EDUCATION
-# represented here is a marginal dependence, i.e., it describes the behavior
+# represented here is a marginal dependence, i.e. it describes the behavior
 # of a specific variable without keeping the others fixed.
 #
 # Also, the EXPERIENCE and AGE are strongly linearly correlated.
@@ -130,7 +128,7 @@ survey.data.info()
 # In particular categorical variables cannot be included in linear model if not
 # coded as integers first. In addition, to avoid categorical features to be
 # treated as ordered values, we need to one-hot-encode them.
-# Our pre-processor will
+# Our pre-processor will:
 #
 # - one-hot encode (i.e., generate a column by category) the categorical
 #   columns, only for non-binary categorical variables;
@@ -150,8 +148,8 @@ preprocessor = make_column_transformer(
 )
 
 # %%
-# To describe the dataset as a linear model we use a ridge regressor
-# with a very small regularization and to model the logarithm of the WAGE.
+# We use a ridge regressor
+# with a very small regularization to model the logarithm of the WAGE.
 
 from sklearn.compose import TransformedTargetRegressor
 from sklearn.linear_model import Ridge
@@ -173,9 +171,9 @@ model = make_pipeline(
 model.fit(X_train, y_train)
 
 # %%
-# Then we check the performance of the computed model plotting its predictions
-# on the test set and computing,
-# for example, the median absolute error of the model.
+# Then we check the performance of the computed model by plotting its predictions
+# against the actual values on the test set, and by computing
+# the median absolute error.
 
 from sklearn.metrics import PredictionErrorDisplay, median_absolute_error
 
@@ -291,11 +289,12 @@ plt.subplots_adjust(left=0.3)
 # %%
 # Now that the coefficients have been scaled, we can safely compare them.
 #
-# .. warning::
+# .. note::
 #
 #   Why does the plot above suggest that an increase in age leads to a
-#   decrease in wage? Why the :ref:`initial pairplot
-#   <marginal_dependencies>` is telling the opposite?
+#   decrease in wage? Why is the :ref:`initial pairplot
+#   <marginal_dependencies>` telling the opposite?
+#   This difference is the difference between marginal and conditional dependence.
 #
 # The plot above tells us about dependencies between a specific feature and
 # the target when all other features remain constant, i.e., **conditional
@@ -389,8 +388,8 @@ plt.subplots_adjust(left=0.3)
 #
 # .. _covariation:
 
-plt.ylabel("Age coefficient")
-plt.xlabel("Experience coefficient")
+plt.xlabel("Age coefficient")
+plt.ylabel("Experience coefficient")
 plt.grid(True)
 plt.xlim(-0.4, 0.5)
 plt.ylim(-0.4, 0.5)
@@ -401,7 +400,7 @@ _ = plt.title("Co-variations of coefficients for AGE and EXPERIENCE across folds
 # Two regions are populated: when the EXPERIENCE coefficient is
 # positive the AGE one is negative and vice-versa.
 #
-# To go further we remove one of the 2 features and check what is the impact
+# To go further we remove one of the two features, AGE, and check what is the impact
 # on the model stability.
 
 column_to_drop = ["AGE"]
@@ -471,8 +470,7 @@ model.fit(X_train, y_train)
 
 # %%
 # Again, we check the performance of the computed
-# model using, for example, the median absolute error of the model and the R
-# squared coefficient.
+# model using the median absolute error.
 
 mae_train = median_absolute_error(y_train, model.predict(X_train))
 y_pred = model.predict(X_test)
@@ -508,10 +506,7 @@ plt.axvline(x=0, color=".5")
 plt.subplots_adjust(left=0.3)
 
 # %%
-# We now inspect the coefficients across several cross-validation folds. As in
-# the above example, we do not need to scale the coefficients by the std. dev.
-# of the feature values since this scaling was already
-# done in the preprocessing step of the pipeline.
+# We now inspect the coefficients across several cross-validation folds.
 
 cv_model = cross_validate(
     model,
@@ -629,8 +624,8 @@ coefs = pd.DataFrame(
 )
 
 # %%
-plt.ylabel("Age coefficient")
-plt.xlabel("Experience coefficient")
+plt.xlabel("Age coefficient")
+plt.ylabel("Experience coefficient")
 plt.grid(True)
 plt.xlim(-0.4, 0.5)
 plt.ylim(-0.4, 0.5)
@@ -770,9 +765,6 @@ plt.subplots_adjust(left=0.3)
 # * Coefficients must be scaled to the same unit of measure to retrieve
 #   feature importance. Scaling them with the standard-deviation of the
 #   feature is a useful proxy.
-# * Interpreting causality is difficult when there are confounding effects. If
-#   the relationship between two variables is also affected by something
-#   unobserved, we should be careful when making conclusions about causality.
 # * Coefficients in multivariate linear models represent the dependency
 #   between a given feature and the target, **conditional** on the other
 #   features.
@@ -782,7 +774,6 @@ plt.subplots_adjust(left=0.3)
 #   coefficients could significantly vary from one another.
 # * Inspecting coefficients across the folds of a cross-validation loop
 #   gives an idea of their stability.
-# * Coefficients are unlikely to have any causal meaning. They tend
-#   to be biased by unobserved confounders.
-# * Inspection tools may not necessarily provide insights on the true
-#   data generating process.
+# * Interpreting causality is difficult when there are confounding effects. If
+#   the relationship between two variables is also affected by something
+#   unobserved, we should be careful when making conclusions about causality.
